;*******************************************************************************************
;* Title: dimmer.asm                                    Copyright (c) Freescale 2004
;*******************************************************************************************
;* Author: Marcus Espindola - Freescale SPS/BSTC
;*
;* Description: Implementing a Lamp Dimmer with HC908Qx MCU.
;*
;* Documentation: HC908QY4 Data Sheet (MC68HC908QY4/D) for register and bit explanations
;*
;* Include Files: dimmer.equ, MC68HC908QT4.equ
;*
;* Assembler: P&E Microcomputer Systems - CASM for HC08
;*            Metrowerks CodeWarrior Compiler for HC08 V-5.0.17
;*
;* Revision History:
;* Rev #      Date      Who         Comments
;* -----   -----------  ---------   -------------------------------------------------------
;* 0.3      10-Sep-04   Espindola   Adjusted timer values according to circuit
;* 0.2      15-May-04   Espindola   Included 1 minute timer
;* 0.1      09-Feb-04   Espindola   Initial data entry
;*******************************************************************************************
;*******************************************************************************************
;* Freescale reserves the right to make changes without further notice to any product
;* herein to improve reliability, function, or design. Freescale does not assume any
;* liability arising out of the application or use of any product, circuit, or software
;* described herein; neither does it convey any license under its patent rights nor the
;* rights of others. Freescale products are not designed, intended, or authorized for
;* use as components in systems intended for surgical implant into the body, or other
;* applications intended to support life, or for any other application in which the
;* failure of the Freescale product could create a situation where personal injury or
;* death may occur. Should Buyer purchase or use Freescale products for any such
;* intended or unauthorized application, Buyer shall indemnify and hold Freescale and
;* its officers, employees, subsidiaries, affiliates, and distributors harmless against
;* all claims, costs, damages, and expenses, and reasonable attorney fees arising out
;* of, directly or indirectly, any claim of personal injury or death associated with
;* such unintended or unauthorized use, even if such claim alleges that Freescale was
;* negligent regarding the design or manufacture of the part.
;*
;* Freescale is a registered trademark of Freescale Semiconductor, Inc.
;*******************************************************************************************


;					XDEF Entry,main,trim_val

;*******************************************************************************************
;* Equates and Data Table Includes
;*******************************************************************************************

            include 'MC68HC908QT4.equ'    ; For the QT1, QT2, QT4, QY1, QY2, QY4

           org $FFC0

trim_val:  DC.B $FF     ; here we set the FLASH trim to a default value.
                        ; DO NOT change this value, as the trim will not be 
                        ; automatically calibrated by the programming interface if 
                        ; this value is anything other than $FF

            org   RamStart

;*******************************************************************************************
;* Constants and Variables for this file
;*******************************************************************************************

            include 'dimmer.equ'

;*******************************************************************************************
;* SUBROUTINES
;* This part includes subroutines
;*******************************************************************************************

            org   FlashStart

;*******************************************************************************************
;* Table used for timer value after zero-crossing detection
;* This table uses indexed addressing mode
;*******************************************************************************************

LSBTimer:   dc.b  $FF;
            dc.b  $1C;
            dc.b  $73;
            dc.b  $90;
            dc.b  $AD;
            dc.b  $CA;
            dc.b  $E7;
            dc.b  $04;
            dc.b  $21;
            dc.b  $3E;
            dc.b  $5B;
            dc.b  $78;
            dc.b  $95;
            dc.b  $B2;
            dc.b  $CF;
            dc.b  $EC;
            dc.b  $09;
            dc.b  $26;
            dc.b  $43;
            dc.b  $60;
            dc.b  $7D;
            dc.b  $9A;
            dc.b  $B7;
            dc.b  $D4;
            dc.b  $F1;
            dc.b  $0E;
            dc.b  $2B;
            dc.b  $48;
            dc.b  $65;
            dc.b  $82;
            dc.b  $9F;
            dc.b  $BC;
            dc.b  $D9;
            dc.b  $F6;

MSBTimer:   dc.b  $00;
            dc.b  $02;
            dc.b  $03;
            dc.b  $04;
            dc.b  $05;
            dc.b  $06;
            dc.b  $07;
            dc.b  $09;
            dc.b  $0A;
            dc.b  $0B;
            dc.b  $0C;
            dc.b  $0D;
            dc.b  $0E;
            dc.b  $0F;
            dc.b  $10;
            dc.b  $11;
            dc.b  $13;
            dc.b  $14;
            dc.b  $15;
            dc.b  $16;
            dc.b  $17;
            dc.b  $18;
            dc.b  $19;
            DC.B  $1A;
            dc.b  $1B;
            dc.b  $1D;
            dc.b  $1E;
            dc.b  $1F;
            dc.b  $20;
            dc.b  $21;
            dc.b  $22;
            dc.b  $23;
            dc.b  $24;
            dc.b  $25;

InitTimer:  mov   #initTim,TSC ;Timer - Cleared + Stopped.

            mov   #InitTMODH,TMODH ;Set max. brightness
            mov   #InitTMODL,TMODL ;after we start the timer.

            bra   Skip

;Subroutine for Thyristor gate control

Gate:       lda   #GateVal    ;Gate pulse duration
loop:       bset  PTA1,PTA
            nop
            bclr  PTA1,PTA
            dbnza loop

            jmp  Skip
            
;Subroutine for Timer Overflow

TOverflow:  nop
            nop
            brclr TOF,TSC,TOverflow ;Wait for Timer Overflow
       
            lda   TSC
            and   #TSCClr
            sta   TSC         ;Clear TOF bit

            mov   #initTim,TSC ;STOP and RESET Counter

            bra   Skip

;Subroutine for Dimmer

IncTimer:   incx
            cpx   #IncTcomp
            bhi   Escape

            lda   MSBTimer,x
            sta   TMODH
            lda   LSBTimer,x
            sta   TMODL

            bra   Skip
            
Escape:     ldx   #IncTcomp

            lda   MSBTimer,x
            sta   TMODH
            lda   LSBTimer,x
            sta   TMODL

            bra   Skip

DecTimer:   decx
            cpx   #DecTcomp
            blo   EscapeDec

            lda   MSBTimer,x
            sta   TMODH
            lda   LSBTimer,x
            sta   TMODL

            bra   Skip
            
EscapeDec:  ldx   #DecTcomp

            lda   MSBTimer,x
            sta   TMODH
            lda   LSBTimer,x
            sta   TMODL
            
            bra   Skip
                        
Delay:      lda   #Delval
Xloop:      brn   *
            brn   *
            dbnza Xloop

Skip:       rts

      
;*******************************************************************************************
;* Main Init
;* This is the point where code starts executing after a RESET.                
;*******************************************************************************************

main:
            mov   #initCfg1,CONFIG1 ;Set config1 register
                              ;(LVI and COP disabled)

            mov   #initCfg2,CONFIG2 ;set MCU to internal oscillator, IRQ enabled
            
;            mov   #InitIRQ,INTSCR ;Configure IRQ function
;            bset  ACK1,INTSCR ;Clears IRQF1 bit
       
            mov   #InitDDRA,DDRA ;PTA0 -> Zero Crossing detection
            bset  DDRA1,DDRA  ;PTA1 -> Pulses on Thyristor gate
                              ;PTA2 as IRQb -> Turns on dimmer
                              ;PTA3 as RSTb -> Turns on 1-minute timer
                              ;PTA4 -> Dec. lamp brightness
                              ;PTA5 -> Inc. lamp brightness
            bset  PTAPUE4,PTAPUE
            bset  PTAPUE5,PTAPUE

            clr   Counter1
            clr   Counter2
            
            clrh
;            clrx
            ldx   #$01
            
            jsr   InitTimer   ;Goes config Timer
            cli               ;Allow interrupts to happen

Waitpta0:   nop
            brclr PTA0,PTA,Waitpta0  ;Wait for an edge on PTA0 (Zero crossing)
            
            brclr  PTA5,PTA,IncT ;Inc timer if PTA5 is clear
            brclr  PTA4,PTA,DecT ;Dec timer if PTA4 is clear

Next:       mov   #StartTim,TSC  ;Start the timer
            
            jsr   TOverflow   ;Go to Timer Overflow subroutine

            jsr   Gate        ;Go to Gate subroutine
            
Waitpta:    nop
            brset  PTA0,PTA,Waitpta  ;Wait for an edge on PTA0 (Zero crossing)
            
Next1:      mov   #StartTim,TSC ;Start the timer

            jsr   TOverflow   ;Go to Timer Overflow subroutine

            jsr   Gate        ;Go to Gate subroutine
            
            bra   Waitpta0
            
IncT:       jsr   Delay
            cpx   #IncTcomp
            beq   Next
            jsr   IncTimer
            bra   Next
            
DecT:       jsr   Delay
            cpx   #DecTcomp
            beq   Next
            jsr   DecTimer
            bra   Next

;**** Interrupt Routine ***********

;IntIRQ:     bset  ACK1,INTSCR ;Clears IRQF1 bit
            rti
            
;**** Interrupt Vectors ***********

;            org    $FFFA
;            dcw    IntIRQ
            org    $FFFE
            dcw    main

            
END