;*******************************************************************************************
;* Title: minuteria.asm                                    Copyright (c) Freescale 2004
;*******************************************************************************************
;* Author: Marcus Espindola - Freescale SPS/BSTC
;*
;* Description: Implementing a Lamp Dimmer with HC908Qx MCU.
;*
;* Documentation: HC908QY4 Data Sheet (MC68HC908QY4/D) for register and bit explanations
;*
;* Include Files: dimmer.equ, MC68HC908QT4.equ
;*
;* Assembler: P&E Microcomputer Systems - CASM for HC08
;*            Metrowerks CodeWarrior Compiler for HC08 V-5.0.17
;*
;* Revision History:
;* Rev #      Date      Who         Comments
;* -----   -----------  ---------   -------------------------------------------------------
;* 0.3      10-Sep-04   Espindola   Adjusted timer values according to circuit
;* 0.2      15-May-04   Espindola   Included 1 minute timer
;* 0.1      09-Feb-04   Espindola   Initial data entry
;*******************************************************************************************
;*******************************************************************************************
;* Freescale reserves the right to make changes without further notice to any product
;* herein to improve reliability, function, or design. Freescale does not assume any
;* liability arising out of the application or use of any product, circuit, or software
;* described herein; neither does it convey any license under its patent rights nor the
;* rights of others. Freescale products are not designed, intended, or authorized for
;* use as components in systems intended for surgical implant into the body, or other
;* applications intended to support life, or for any other application in which the
;* failure of the Freescale product could create a situation where personal injury or
;* death may occur. Should Buyer purchase or use Freescale products for any such
;* intended or unauthorized application, Buyer shall indemnify and hold Freescale and
;* its officers, employees, subsidiaries, affiliates, and distributors harmless against
;* all claims, costs, damages, and expenses, and reasonable attorney fees arising out
;* of, directly or indirectly, any claim of personal injury or death associated with
;* such unintended or unauthorized use, even if such claim alleges that Freescale was
;* negligent regarding the design or manufacture of the part.
;*
;* Freescale is a registered trademark of Freescale Semiconductor, Inc.
;*******************************************************************************************


;					XDEF Entry,main,trim_val

;*******************************************************************************************
;* Equates and Data Table Includes
;*******************************************************************************************

            include 'MC68HC908QT4.equ'    ; For the QT1, QT2, QT4, QY1, QY2, QY4

           org $FFC0

trim_val:  DC.B $FF     ; here we set the FLASH trim to a default value.
                        ; DO NOT change this value, as the trim will not be 
                        ; automatically calibrated by the programming interface if 
                        ; this value is anything other than $FF

            org   RamStart

;*******************************************************************************************
;* Constants and Variables for this file
;*******************************************************************************************

            include 'dimmer.equ'

;*******************************************************************************************
;* SUBROUTINES
;* This part includes subroutines
;*******************************************************************************************

            org   FlashStart

InitTimer:  mov   #initTim,TSC ;Timer - Cleared + Stopped.

            mov   #InitTMODH,TMODH ;Set max. brightness
            mov   #InitTMODL,TMODL ;after we start the timer.

            bra   Skip

;Subroutine for Thyristor gate control

Gate:       lda   #GateVal    ;Gate pulse duration
loop:       bset  PTA1,PTA
            nop
            bclr  PTA1,PTA
            dbnza loop

            jmp  Skip
            
;Subroutine for Timer Overflow

TOverflow:  nop
            nop
            brclr TOF,TSC,TOverflow ;Wait for Timer Overflow
       
            lda   TSC
            and   #TSCClr
            sta   TSC         ;Clear TOF bit

            mov   #initTim,TSC ;STOP and RESET Counter

            bra   Skip

Skip:       rts

      
;*******************************************************************************************
;* Main Init
;* This is the point where code starts executing after a RESET.                
;*******************************************************************************************

main:
            mov   #initCfg1,CONFIG1 ;Set config1 register
                              ;(LVI and COP disabled)

            mov   #initCfg2,CONFIG2 ;set MCU to internal oscillator, IRQ enabled
            
;            mov   #InitIRQ,INTSCR ;Configure IRQ function
;            bset  ACK1,INTSCR ;Clears IRQF1 bit
       
            mov   #InitDDRA,DDRA ;PTA0 -> Zero Crossing detection
            bset  DDRA1,DDRA  ;PTA1 -> Pulses on Thyristor gate
                              ;PTA3 as RSTb -> Turns on 1-minute timer

            clr   Counter1
            clr   Counter2
            
            clrh
            clrx
            
            jsr   InitTimer   ;Goes config Timer
            cli               ;Allow interrupts to happen

ZeroDetec:  nop
            brclr PTA0,PTA,ZeroDetec ;Wait for a edge on PTA0 (Zero crossing)
            
            mov   #StartTim,TSC  ;Start the timer
            
            jsr   TOverflow   ;Go to Timer Overflow subroutine

            jsr   Gate        ;Go to Gate subroutine
            
ZeroDetect: nop
            brset  PTA0,PTA,ZeroDetect ;Wait for a edge on PTA0 (Zero crossing)
            
            mov   #StartTim,TSC ;Start the timer

            jsr   TOverflow   ;Go to Timer Overflow subroutine

            jsr   Gate        ;Go to Gate subroutine
            
            inc   Counter1    ;Increment 1st byte Counter for charge time OVF period
            lda   #CntLcmp
            cbeq  Counter1,Count1

            bra   ZeroDetec

Count1:     inc   Counter2    ;Increment 2nd byte Counter for charge time OVF period
            lda   #CntHcmp
            cbeq  Counter2,Out

            bra   ZeroDetec

Out:        clr   Counter1
            clr   Counter2
            stop
            
;**** Interrupt Vectors ***********

            org    $FFFE
            dcw    main

END